import os

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from matplotlib import colors as mcolors

# constants
bar_width = 0.2
solver_names = ['ilp-direct', 'simple']
patterns = ['❌', '✔']

colors = [c[4:] for c in (sorted(mcolors.TABLEAU_COLORS.keys())) if 'dark' + c[4:] in mcolors.CSS4_COLORS]
colors.reverse()


def to_label(name):
    if '-' in name:
        tokens = name.split('-')
        return '{} ({})'.format(tokens[0].upper(), tokens[1].title())
    else:
        return name.title()


def load(name, show_head=False):
    data = pd.read_csv(name)
    data['tool'] = data.name.astype(str)
    data['Gen'] = (data.Gen.astype(int) > 0) * data.Gen.astype(int)
    data['total'] = data.Gen.astype(int) + data.Solved.astype(int)
    if show_head:
        data.head()
    return data


def load_scenario_file(name, show_head=False):
    for filename in os.listdir("scenarios"):
        if filename.endswith(name):
            return load("scenarios/" + filename, show_head)


def plot_range(names, labels, ax):
    # constants

    # colors = plt.get_cmap("rainbow")
    #     colors = list(mcolors.to_rgba_array(color))
    N = len(solver_names)
    width = 0.05
    ind = np.arange(N)

    barpos = []
    for i in range(len(names)):
        barpos += list((ind + i * (N + 0.5)) * width)

    labelpos = []
    for i in range(len(names)):
        labelpos += [(0.5 + i * (N + 0.5)) * width]

    for (name, i) in zip(names, range(len(names))):

        # load data
        data = load_scenario_file(name)
        means = data.total

        rect = ax.bar((ind + i * (N + 0.5)) * width, means, width=width, color=colors)
        for r, isValid, color in zip(rect, data.Valid, colors):
            ax.text(r.get_x() + r.get_width() / 2,
                    r.get_height() * 1.4,
                    patterns[isValid],  # the text
                    fontname='symbola', fontsize=16, ha='center', va='bottom', color='black')

        for r, isValid, color in zip(rect, data.Valid, colors):
            ax.text(r.get_x() + r.get_width() / 2,
                    r.get_height(),
                    '%d' % int(r.get_height()),
                    fontsize=8, ha='center', va='bottom', color='black')



    ax.set_xticks(labelpos)
    ax.set_xticklabels(labels)
    # ax.legend(rect, [to_label(n) for n in solver_names])
    ax.set_yticks([1, 1000, 1000000])


def create_grid():
    variants = [2, 4]
    requests = [1, 2, 3]
    depth = [1, 2, 3]
    resources = [15, 30]

    y_dimension = (requests, "Requests")
    x_dimension = (variants, "Implementation\nVariants")
    inner_dimension = (depth, "Depth")
    fixed_dimension = (resources, "Resources")
    fixed_dimension_fix = 0

    pos_map = {
        "Implementation\nVariants": 0,
        "Requests": 1,
        "Depth": 2,
        "Resources": 3
    }



    fig, axs = plt.subplots(len(y_dimension[0]), len(x_dimension[0]), figsize=(12, 12), subplot_kw=dict(yscale="log"))

    for y, y_pos in zip(y_dimension[0], range(len(y_dimension[0]))):
        for x, x_pos in zip(x_dimension[0], range(len(x_dimension[0]))):
            parameters = []
            labels = []
            for inner in inner_dimension[0]:
                new_parameters = [0,0,0,0]
                new_parameters[pos_map[x_dimension[1]]] = x
                new_parameters[pos_map[y_dimension[1]]] = y
                new_parameters[pos_map[inner_dimension[1]]] = inner
                new_parameters[pos_map[fixed_dimension[1]]] = fixed_dimension[0][fixed_dimension_fix]
                parameters += [new_parameters]
                labels += [inner_dimension[1] + ' = %d' % inner]
            print(parameters)
            plot_range(['size_v%d_q%d_d%d_r%d.csv' % (p[0], p[1], p[2], p[3]) for p in parameters], [label for label in labels],
                       axs[y_pos, x_pos])

    for i in range(len(y_dimension[0])):
        axs[i][-1].set_ylabel(y_dimension[1] + ' = ' + str(y_dimension[0][i]), rotation=0)
        axs[i][-1].yaxis.set_label_coords(1.2, 0.5)

    for i in range(len(x_dimension[0])):
        axs[0][i].set_xlabel(x_dimension[1] + ' = ' + str(x_dimension[0][i]), rotation=0)
        axs[0][i].xaxis.set_label_coords(0.5, 1.05)

    fig.patch.set_facecolor('w')

    plt.show()


create_grid()
